/*
    Copyright (C) 2001  bjk <bjk@arbornet.org>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <utmp.h>
#include <errno.h>
#include <fcntl.h>
#include "common.h"

#define UTMP_LINE 1
#define UTMP_NAME 2
#define UTMP_HOST 3
#define UTMP_TIME 4
#define DURATION 5

static char *lastlogin(INFO *info)
{
    int count;
    long offset;
    char *str = NULL, buf[UT_LINESIZE + UT_HOSTSIZE + TIMEBUF + 2];
    char tmp[TIMEBUF], htmp[UT_HOSTSIZE];
    struct lastlog last;

    buf[0] = tmp[0] = htmp[0] = 0;

    if (!lastfp) {
	if ((lastfp = fopen(_PATH_LASTLOG, "r")) == NULL)
	    return UNKNOWN;
    }

    offset = (long)info->passwd->pw_uid * sizeof(struct lastlog);

    if (fseek(lastfp, offset, SEEK_SET) == -1) {
	perror(_PATH_LASTLOG);
	return UNKNOWN;
    }

    if ((count = fread(&last, sizeof(last), 1, lastfp)) < 1) {
	perror(_PATH_LASTLOG);
	return UNKNOWN;
    }

    if (last.ll_line[0] == 0)
	strncpy(buf, NONE, sizeof(buf));
    else
	strncpy(buf, last.ll_line, sizeof(buf));

    strncat(buf, ",", sizeof(buf));

    strncpy(htmp, last.ll_host, sizeof(htmp));
    htmp[UT_HOSTSIZE - 1] = 0;

    if (htmp[0] == 0)
	strncat(buf, NONE, sizeof(buf));
    else
	strncat(buf, htmp, sizeof(buf));

    strncat(buf, ",", sizeof(buf));

    if (last.ll_time)
	strncat(buf, stamp(last.ll_time, tf), sizeof(buf));
    else
	strncat(buf, NONE, sizeof(buf));

    str = buf;
    return str;
}

static char *utmpfud(INFO *info, int opt)
{
    int count;
    char *str = NULL;
    time_t epoch;
    struct utmp myutmp;

    if (!utmpfd) {
        if ((utmpfd = open(_PATH_UTMP, O_RDONLY)) < 0) {
	    perror(_PATH_UTMP);
	    return NULL;
        }
    }

    if (lseek(utmpfd, 0, SEEK_SET) == -1) {
	perror(_PATH_UTMP);
	return NULL;
    }

    while ((count = read(utmpfd, &myutmp, sizeof(struct utmp))) == 
	    sizeof(struct utmp)) {
	char *tmp = NULL;

	if (strncmp(info->passwd->pw_name, myutmp.ut_name,
		    sizeof(myutmp.ut_name)) == 0) {
	    info->login->online = 1;

	    switch (opt) {
		case UTMP_LINE:
		    str = myutmp.ut_line;
		    break;
		case UTMP_NAME:
		    str = myutmp.ut_name;
		    break;
		case UTMP_HOST:
		    if (myutmp.ut_host[0] != 0) {
		        if ((tmp = calloc(UT_HOSTSIZE, sizeof(char))) == NULL) {
			    perror("calloc");
			    exit(errno);
		        }

		        strncpy(tmp, myutmp.ut_host, UT_HOSTSIZE);
		        tmp[UT_HOSTSIZE - 1] = 0;
			str = tmp;
		        free(tmp);
		    }
		    else
			str = UNKNOWN;
		    break;
		case UTMP_TIME:
		    str = stamp(myutmp.ut_time, tf);
		    break;
		case DURATION:
		    time(&epoch);

		    if ((epoch - myutmp.ut_time) > 60)
			str = itoa((epoch - myutmp.ut_time) / 60);

		    if (!str)
			str = UNKNOWN;
		    break;
		default:
		    break;
	    }

	    if (str)
		break;
	}
    }
    
    return str;
}

static int msgstat(INFO *info)
{
    char filename[NAME_MAX];
    int i = 0;
    struct stat mystat;

    strncpy(filename, "/dev/", sizeof(filename));
    strncat(filename, utmpfud(info, UTMP_LINE), sizeof(filename));

    if (stat(filename, &mystat) == -1) 
	return 0;

    if (mystat.st_mode & S_IWGRP || mystat.st_mode & S_IWOTH)
	i++;

    return i;
}

void logininfo(INFO *info, const char *arg)
{
    int i;

    if ((info->login = (LOGIN *) malloc(sizeof(LOGIN))) == NULL) {
	perror("malloc");
	exit(errno);
    }

    bzero(info->login, sizeof(LOGIN));
    (void *) utmpfud(info, 0);

    for (i = 0; i < MAXOPTIONS; i++) {
	if (login_opts[i] == 0)
	    break;

	switch (login_opts[i]) {
	    case 'n':
		strncpy(info->login->last, lastlogin(info), 
			sizeof(info->login->last));
		break;
	    case 'x':
		if (info->login->online)
		    strncpy(info->login->host, utmpfud(info, UTMP_HOST),
			    sizeof(info->login->host));
		else
		    strncpy(info->login->host, UNKNOWN,
			    sizeof(info->login->host));
		break;
	    case 'y':
		if (info->login->online)
		    strncpy(info->login->tty, utmpfud(info, UTMP_LINE), 
			    sizeof(info->login->tty));
		else
		    strncpy(info->login->tty, UNKNOWN,
			    sizeof(info->login->tty));
		break;
	    case 'e':
		if (info->login->online)
		    info->login->mesgstat = msgstat(info);
		break;
	    case 't':
		if (info->login->online)
		    strncpy(info->login->stamp, utmpfud(info, UTMP_TIME),
			    sizeof(info->login->stamp));
		else
		    strncpy(info->login->stamp, UNKNOWN, 
			    sizeof(info->login->stamp));
		break;
	    case 'b':
		if (info->login->online)
		    strncpy(info->login->duration, utmpfud(info, DURATION),
			    sizeof(info->login->duration));
		else
		    strncpy(info->login->duration, UNKNOWN,
			    sizeof(info->login->duration));
		break;
	    default:
		break;
	}
    }

    return;
}
